//---Include---
#include "Text.h"
#include <sstream>
#include <iomanip>
#ifndef WIN32
#include <string.h>
#endif

using namespace Framework;

// inhalt der Text Klasse aus Text.h
// Konstruktor 
Text::Text()
    : txt( 0 ),
    suchGBeg( 0 ),
    suchGEnd( 0 ),
    precision( 0 ),
    ref( 1 )
{
    setText( "" );
}

Text::Text( const Text &txt )
    : txt( 0 ),
    suchGBeg( 0 ),
    suchGEnd( 0 ),
    precision( 0 ),
    ref( 1 )
{
    setText( txt );
}

Text::Text( const char *t )
    : txt( 0 ),
    suchGBeg( 0 ),
    suchGEnd( 0 ),
    precision( 0 ),
    ref( 1 )
{
    setText( t ); // Text setzen
}

Text::Text( int zahl )
    : txt( 0 ),
    suchGBeg( 0 ),
    suchGEnd( 0 ),
    precision( 0 ),
    ref( 1 )
{
    *this = zahl;
}

// Erstellt ein neues Text Objekt mit einer zahl als text
//  num: Die Zahl, die im Text sein soll
Text::Text( double num )
    : txt( 0 ),
    suchGBeg( 0 ),
    suchGEnd( 0 ),
    precision( 0 ),
    ref( 1 )
{
    *this = num;
}

// Destruktor 
Text::~Text()
{
    delete[]txt;
}

// nicht constant
void Text::setSuchGrenzen( char gBeg, char gEnd ) // sucht bei jeder suchfunktion nicht zwischen den Zeichen gBeg und gEnd
{
    suchGBeg = gBeg;
    suchGEnd = gEnd;
}

void Text::setText( const char *t ) // ersetzt den Text
{
    delete[]txt; // alter Text lschen
    int l = (int)strlen( t ); // Lnge des neuen Textes ermitteln
    txt = new char[ l + 1 ]; // neuen Text erstellen
    for( int i = 0; i < l; ++i ) // Text befllen
        txt[ i ] = t[ i ];
    txt[ l ] = '\0'; // Textende Festlegen
}

// unconstant 
void Text::setText( const char *t, int l ) // ersetzt den Text
{
    delete[]txt; // alter Text lschen
    txt = new char[ l + 1 ]; // neuen Text erstellen
    for( int i = 0; i < l; ++i ) // Text befllen
        txt[ i ] = t[ i ];
    txt[ l ] = '\0'; // Textende Festlegen
}

void Text::setText( Text *t )
{
    setText( t->getText() ); // Text setzen
    t = t->release(); // bergabe loslassen
}

void Text::append( char c ) // hngt an den Text an
{
	append( &c, 1 );
}

void Text::append( const char *t ) // hngt an den Text an
{
    int tl = (int)strlen( t ); // lnge der bergabe
    int txl = getLength(); // lnge des Textes
    char *res = new char[ tl + txl + 1 ]; // neuen Text erstellen
    for( int i = 0; i < txl; ++i ) // mit jetzigem Text fllen
        res[ i ] = txt[ i ];
    for( int i = 0; i < tl; ++i ) // bergabe anhngen
        res[ txl + i ] = t[ i ];
    res[ txl + tl ] = '\0'; // Textende festlegen
    setText( res ); // Test setzen
    delete[] res; // Speicher freigeben
}

void Text::appendHex( int num ) // hngt die zahl in hex anden Text an
{
    int l = getLength();
    char *res = new char[ l + 9 ];
    for( int i = 0; i < l; ++i )
        res[ i ] = txt[ i ];
    std::stringstream stream;
    stream << std::setfill( '0' ) << std::setw( (int)sizeof( int ) * 2 ) << std::hex << num;
    std::string str = stream.str();
    for( int i = l; i < l + 8; ++i )
        res[ i ] = str.c_str()[ i - l ];
    res[ l + 8 ] = 0;
    setText( res );
    delete[] res;
}

void Text::appendColor( int fc ) // setzt die Farbe des folgenden Textes
{
    int l = getLength();
    char *res = new char[ l + 12 ];
    for( int i = 0; i < l; ++i )
        res[ i ] = txt[ i ];
    res[ l ] = '\r';
    res[ l + 1 ] = '0';
    res[ l + 2 ] = 'x';
    std::stringstream stream;
    stream << std::setfill( '0' ) << std::setw( (int)sizeof( int ) * 2 ) << std::hex << fc;
    std::string str = stream.str();
    for( int i = l + 3; i < l + 11; ++i )
        res[ i ] = str.c_str()[ i - ( l + 3 ) ];
    res[ l + 11 ] = 0;
    setText( res );
    delete[] res;
}

void Text::append( const char *t, int l ) // hngt an den Text an
{
    int txl = getLength(); // lnge des Textes
    char *res = new char[ l + txl + 1 ]; // neuen Text erstellen
    for( int i = 0; i < txl; ++i ) // mit jetzigem Text fllen
        res[ i ] = txt[ i ];
    for( int i = 0; i < l; ++i ) // bergabe anhngen
        res[ txl + i ] = t[ i ];
    res[ txl + l ] = '\0'; // Textende festlegen
    setText( res, txl + l ); // Test setzen
    delete[] res; // Speicher freigeben
}

void Text::append( Text *t )
{
    if( t->getLength() > 0 ) // Auf unsinnige bergabe prfen
        append( t->getText() ); // Text anhngen
    t = t->release(); // bergabe loslassen
}

void Text::append( int num )
{
    std::stringstream ss;
    ss << num;
    append( ss.str().c_str() );
}

// Hngt eine Zahl ans Ende Des Textes an
//  num: Die Zahl, die in einen Text umgewandelt und am Ende angehngt werden soll
void Text::append( __int64 num )
{
    std::stringstream ss;
    ss << num;
    append( ss.str().c_str() );
}

void Text::append( unsigned int num )
{
    std::stringstream ss;
    ss << num;
    append( ss.str().c_str() );
}

void Text::append( double num )
{
    std::stringstream ss;
    ss.precision( precision );
    ss << num;
    append( ss.str().c_str() );
}

void Text::insert( int p, char c ) // Fgt an stelle p ein
{
    if( p > getLength() || p < 0 ) // Auf unsinnige bergabe prfen
        return;
    int txl = getLength(); // Lnge des Textes
    char *res = new char[ txl + 2 ]; // neuer Text erstellen
    for( int i = 0; i < p; ++i ) // Text fllen
        res[ i ] = txt[ i ];
    res[ p ] = c;
    for( int i = p; i < txl; ++i ) // Text fllen
        res[ i + 1 ] = txt[ i ];
    res[ txl + 1 ] = '\0'; // Text ende festlegen
    setText( res ); // Text setzen
    delete[] res; // Speicher freigeben
}

void Text::insert( int p, const char *t ) // Fgt an stelle p ein
{
    if( p > getLength() || p < 0 || strlen( t ) <= 0 ) // Auf unsinnige bergabe prfen
        return;
    int tl = (int)strlen( t ); // Lnge der bergabe
    int txl = getLength(); // Lnge des Textes
    char *res = new char[ tl + txl + 1 ]; // neuer Text erstellen
    for( int i = 0; i < p; ++i ) // Text fllen
        res[ i ] = txt[ i ];
    for( int i = 0; i < tl; ++i ) // Text einfgen
        res[ i + p ] = t[ i ];
    for( int i = p; i < txl; ++i ) // Text fllen
        res[ i + tl ] = txt[ i ];
    res[ tl + txl ] = '\0'; // Text ende festlegen
    setText( res ); // Text setzen
    delete[] res; // Speicher freigeben
}

void Text::insert( int p, Text *t )
{
    if( t->getLength() > 0 ) // Auf unsinnige bergabe prfen
        insert( p, t->getText() ); // Text einfgen
    t = t->release(); // bergabe loslassen
}

void Text::ersetzen( int p1, int p2, const char *t ) // Ersetzt den Text von p1 bis p2
{
    if( p1 > p2 )
    {
        int x = p1; // p1 und p2 tauschen
        p1 = p2;
        p2 = x;
    }
    remove( p1, p2 ); // Textabschnitt von p1 zu p2 lschen
    insert( p1, t ); // bergabe bei p1 einfgen
}

void Text::ersetzen( int p1, int p2, Text *t )
{
    if( t->getLength() >= 0 ) // Auf unsinnige bergabe prfen
        ersetzen( p1, p2, t->getText() ); // Text ersetzen
    t = t->release(); // bergabe loslassen
}

void Text::ersetzen( char c1, char c2 ) // ersetzt jedes c1 durch c2
{
    if( c1 == '\0' || c2 == '\0' ) // Auf unsinnige bergabe prfen
        return;
    if( !hat( c1 ) ) // prfen ob c1 vorhanden
        return;
    int l = getLength(); // Text Lnge
    int suchGCount = 0;
    for( int i = 0; i < l; ++i ) // Text durchsuchen
    {
        bool b = suchGCount != 0;
        if( txt[ i ] == c1 && !suchGCount )
            txt[ i ] = c2; // Text ersetzen
        if( txt[ i ] == suchGBeg )
            ++suchGCount;
        if( txt[ i ] == suchGEnd )
            --suchGCount;
        if( txt[ i ] == c1 && !suchGCount && b )
            txt[ i ] = c2; // Text ersetzen
    }
}

void Text::ersetzen( const char *t1, const char *t2 ) // ersetzt jedes t1 durch t2
{
    int txl = getLength(); // Text Lnge
    int t1l = (int)strlen( t1 ); // Lnge der bergaben
    int t2l = (int)strlen( t2 );
    if( t1l > txl || t1l <= 0 ) // Auf unsinnige bergabe prfen
        return;
    if( !hat( t1 ) ) // prfen ob t1 vorhanden
        return;
    int anz = anzahlVon( t1 ); // Anzahl von t1 im Text
    int *begin = new int[ anz ];
    int *end = new int[ anz ];
    for( int i = 0; i < anz; ++i ) // Positionen von t1 speichern
    {
        begin[ i ] = positionVon( t1, i );
        end[ i ] = begin[ i ] + t1l;
    }
    int resl = ( txl - ( anz * t1l ) ) + ( anz * t2l ) + 1; // Lnge des Ergebneses
    char *res = new char[ resl ]; // neuer Text erstellen
    int rep = 0; // Speichert bei welchen t1 man sich befindet
    int last = 0; // Fllposition von txt
    int neu = 0; // Fllporition von res
    for( ; neu < resl; ++neu ) // Text befllen
    {
        if( rep < anz && last == begin[ rep ] ) // Text ersetzen
        {
            last = end[ rep ];
            ++rep;
            for( int ii = 0; ii < t2l; ++ii )
            {
                if( neu >= resl )
                    break;
                res[ neu ] = t2[ ii ];
                ++neu;
            }
            if( neu >= resl )
                break;
            --neu;
        }
        else // Text Kopieren
        {
            res[ neu ] = txt[ last ];
            ++last;
        }
    }
    res[ resl - 1 ] = '\0'; // Textende festlegen
    setText( res ); // Text setzen
    delete[]begin; // Speicher freigeben
    delete[]end;
    delete[] res;
}

void Text::ersetzen( Text *t1, const char *t2 )
{
    if( t1->getLength() > 0 ) // Auf unsinnige bergabe prfen
        ersetzen( t1->getText(), t2 ); // ersetzen
    t1 = t1->release(); // bergabe loslassen
}

void Text::ersetzen( const char *t1, Text *t2 )
{
    if( t2->getLength() >= 0 ) // Auf unsinnige bergabe prfen
        ersetzen( t1, t2->getText() ); // ersetzen
    t2 = t2->release(); // bergabe loslassen
}

void Text::ersetzen( Text *t1, Text *t2 )
{
    if( t1->getLength() > 0 && t2->getLength() >= 0 ) // Auf unsinnige bergabe prfen
        ersetzen( t1->getText(), t2->getText() ); // ersetzen
    t1 = t1->release(); // bergaben loslassen
    t2 = t2->release();
}

void Text::ersetzen( int index, char c1, char c2 ) // ersetzt das i-te c1 durch c2
{
    if( c1 == '\0' || c2 == '\0' || index < 0 ) // Auf unsinnige bergabe prfen
        return;
    if( !hat( c1 ) ) // prfen ob c1 vorhanden
        return;
    int anz = anzahlVon( c1 );
    if( index >= anz ) // prfen ob es ein i-tes c1 gibt
        return;
    txt[ positionVon( c1, index ) ] = c2;
}

void Text::ersetzen( int index, const char *t1, const char *t2 ) // ersetzt das i-te t1 durch t2
{
    int txl = getLength(); // Text Lnge
    int t1l = (int)strlen( t1 ); // Lnge der bergaben
    if( t1l >= txl || t1l <= 0 || index < 0 ) // Auf unsinnige bergabe prfen
        return;
    if( !hat( t1 ) ) // prfen ob t1 vorhanden
        return;
    int anz = anzahlVon( t1 ); // Anzahl von t1 im Text
    if( index >= anz ) // prfen ob es ein i-tes t1 gibt
        return;
    int begin = positionVon( t1, index );
    int end = begin + t1l;
    ersetzen( begin, end, t2 ); // ersetzen
}

void Text::ersetzen( int i, Text *t1, const char *t2 )
{
    if( t1->getLength() > 0 ) // Auf unsinnige bergabe prfen
        ersetzen( i, t1->getText(), t2 ); // ersetzen
    t1 = t1->release(); // bergabe loslassen
}

void Text::ersetzen( int i, const char *t1, Text *t2 )
{
    if( t2->getLength() >= 0 ) // Auf unsinnige bergabe prfen
        ersetzen( i, t1, t2->getText() ); // ersetzen
    t2 = t2->release(); // bergabe loslassen
}

void Text::ersetzen( int i, Text *t1, Text *t2 )
{
    if( t1->getLength() > 0 || t2->getLength() >= 0 ) // Auf unsinnige bergabe prfen
        ersetzen( i, t1->getText(), t2->getText() ); // ersetzen
    t1 = t1->release(); // bergaben loslassen
    t2 = t2->release();
}

void Text::fillText( char c, int len ) // setzt den Text zu so vielen c wie ln gro ist
{
    char *res = new char[ len + 1 ];
    for( int i = 0; i < len; ++i )
        res[ i ] = c;
    res[ len ] = '\0';
    setText( res );
    delete[] res;
}

void Text::remove( int p ) // lscht p
{
    int l = getLength(); // Text Lnge
    if( p < 0 || p >= l ) // Auf unsinnige bergabe prfen
        return;
    char *res = new char[ l ]; // neuen Text anlegen
    for( int i = 0; i < p && i < l; ++i ) // Text befllen
        res[ i ] = txt[ i ];
    for( int i = p + 1; i < l; ++i )
        res[ i - 1 ] = txt[ i ];
    res[ l - 1 ] = 0;
    setText( res ); // Text setzen
    delete[] res; // Speicher freigeben
}

void Text::remove( int p1, int p2 ) // lscht von p1 zu p2 ( p2 bleibt )
{
    if( p1 > p2 ) // p1 und p2 tauschen
    {
        int x = p1;
        p1 = p2;
        p2 = x;
    }
    int l = getLength(); // Lnge des Testes
    if( p1 < 0 || p2 > l ) // Auf unsinnige bergabe prfen
        return;
    int resl = l - ( p2 - p1 ); // Lnge vom Ergebnis
    char *res = new char[ resl + 1 ]; // Neuen Text erstellen
    for( int i = 0; i < p1; ++i ) // Text fllen
        res[ i ] = txt[ i ];
    for( int i = p2; i < l; ++i )
        res[ i - ( p2 - p1 ) ] = txt[ i ];
    res[ resl ] = '\0'; // Testende festlegen
    setText( res ); // Text setzen
    delete[] res; // Speicher freigeben
}

void Text::remove( char c ) // lscht jetes c
{
    if( !hat( c ) ) // prfen ob c vorhanden
        return;
    int l = getLength(); // Lnge des Textes
    int anz = anzahlVon( c ); // Anzahl von c
    char *res = new char[ l - anz + 1 ]; // neuen Text erstellen
    int anz2 = 0;
    int suchGCount = 0;
    for( int i = 0; i < l; ++i ) // Text befllen
    {
        bool b = suchGCount != 0;
        if( txt[ i ] == c && !suchGCount )
            ++anz2;
        else
            res[ i - anz2 ] = txt[ i ];
        if( txt[ i ] == suchGBeg )
            ++suchGCount;
        if( txt[ i ] == suchGEnd )
            --suchGCount;
        if( b )
        {
            if( txt[ i ] == c && !suchGCount )
                ++anz2;
            else
                res[ i - anz2 ] = txt[ i ];
        }
    }
    res[ l - anz ] = '\0'; // Textende festlegen
    setText( res ); // Text setzen
    delete[] res; // Speicher freigeben
}

void Text::remove( const char *t ) // lscht jetes t
{
    int tl = (int)strlen( t ); // Lnge der bergabe
    int txl = getLength(); // Lnge des Textes
    if( tl <= 0 || tl > txl ) // Auf unsinnige bergabe prfen
        return;
    if( !hat( t ) ) // prfen ob Text ein t enthlt
        return;
    int anz = anzahlVon( t ); // anzahl von t
    int *begin = new int[ anz ];
    for( int i = 0; i < anz; ++i ) // begin aller t-s finden
        begin[ i ] = positionVon( t, i );
    int resl = txl - ( anz * tl ); // Lnge vom Ergebnes
    char *res = new char[ resl + 1 ]; // neuen Text erzeugen
    int del = 0;
    for( int i = 0; i < txl; ++i ) // Text befllen
    {
        if( del < anz && i == begin[ del ] ) // Text auslassen
        {
            i += tl - 1;
            ++del;
        }
        else
            res[ i - ( del * tl ) ] = txt[ i ]; // Text befllen
    }
    res[ resl ] = '\0'; // Text ende festlegen
    setText( res ); // Text setzen
    delete[]begin;
    delete[] res; // Speicher freigeben
}

void Text::remove( Text *t )
{
    if( t->getLength() > 0 ) // Auf unsinnige bergabe prfen
        remove( t->getText() ); // lschen
    t = t->release(); // bergabe loslassen
}

void Text::remove( int index, char c )
{
    if( index < 0 || !hat( c ) ) // Auf unsinnige bergabe prfen
        return;
    int anz = anzahlVon( c ); // anzahl der i-s
    if( index >= anz ) // prfen ob ein i-tes c vorhanden ist
        return;
    int pos = positionVon( c, index ); // Position vom i-ten c
    if( pos < 0 )
        return;
    int l = getLength(); // Lnge des Textes
    if( !l )
        return;
    char *res = new char[ l ]; // neuen Text erzeugen
    for( int i = 0; i < pos && i < l; ++i ) // Text befllen
        res[ i ] = txt[ i ];
    for( int i = pos + 1; i < l; ++i )
        res[ i - 1 ] = txt[ i ];
    res[ l - 1 ] = '\0'; // Text ende festlegen
    setText( res ); // Text setzen
    delete[] res; // Speicher freigeben
}

void Text::remove( int index, const char *t ) // lscht das i-te t
{
    int tl = (int)strlen( t ); // Lnge der bergabe
    if( index < 0 || !hat( t ) || tl <= 0 ) // Auf unsinnige bergabe prfen
        return;
    int anz = anzahlVon( t ); // anzahl der i-s
    if( index >= anz ) // prfen ob ein i-tes c vorhanden ist
        return;
    int pos = positionVon( t, index ); // Position vom i-ten c
    if( pos < 0 )
        return;
    int l = getLength(); // Lnge des Textes
    if( !l )
        return;
    char *res = new char[ l - tl + 1 ]; // neuen Text erzeugen
    for( int i = 0; i < pos && i < l - tl + 1; ++i ) // Text befllen
        res[ i ] = txt[ i ];
    for( int i = pos + tl; i < l; ++i )
        res[ i - tl ] = txt[ i ];
    res[ l - tl ] = '\0'; // Text ende festlegen
    setText( res ); // Text setzen
    delete[] res; // Speicher freigeben
}

void Text::remove( int i, Text *t )
{
    if( t->getLength() > 0 ) // Auf unsinnige bergabe prfen
        remove( i, t->getText() ); // lschen
    t = t->release(); // bergabe loslassen
}

void Text::setPrecision( int p ) // setzt die Anzahl der Nachkommastellen bei doubles
{
    precision = p;
}

// constant 
int Text::getLength() const // gibt die Text lnge zurck
{
    if( !txt )
        return -1;
    int ret = 0;
    for( ; txt[ ret ] != '\0'; ++ret ); // zhlen
    return ret;
}

int Text::getLKick( int pos ) const
{
    if( txt[ pos - 1 ] == ' ' )
    {
        int ret = 1;
        for( ; ret < pos && txt[ pos - ret - 1 ] == ' ' && txt[ pos - ret - 1 ] != '\n'; ++ret );
        return pos - ret;
    }
    else
    {
        int ret = 1;
        for( ; ret < pos && txt[ pos - ret - 1 ] != ' ' && txt[ pos - ret - 1 ] != '\n'; ++ret );
        return pos - ret;
    }
}

int Text::getOKick( int pos ) const
{
    if( !hat( '\n' ) )
        return 0;
    int lpos = 0;
    while( pos - lpos - 1 > 0 && txt[ pos - lpos - 1 ] != '\n' )
        ++lpos;
    int vllen = 1;
    while( pos - lpos - vllen - 1 >= 0 && txt[ pos - lpos - vllen - 1 ] != '\n' )
        ++vllen;
    if( vllen > lpos )
        return pos - vllen;
    else
        return pos - lpos - 1;
}

int Text::getRKick( int pos ) const
{
    int tl = getLength();
    if( txt[ pos ] == ' ' )
    {
        int ret = 1;
        for( ; ret + pos < tl && txt[ pos + ret ] == ' ' && txt[ pos + ret ] != '\n'; ++ret );
        return pos + ret;
    }
    else
    {
        int ret = 1;
        for( ; ret + pos < tl && txt[ pos + ret ] != ' '&& txt[ pos + ret ] != '\n'; ++ret );
        return pos + ret;
    }
}

int Text::getUKick( int pos ) const
{
    if( !hat( '\n' ) )
        return getLength();
    int lpos = 0;
    while( pos - lpos > 0 && txt[ pos - lpos - 1 ] != '\n' )
        ++lpos;
    int llen = 1;
    while( pos + llen - 1 < getLength() && txt[ pos + llen - 1 ] != '\n' )
        ++llen;
    int vllen = 1;
    while( pos + llen + vllen - 1 < getLength() && txt[ pos + llen + vllen - 1 ] != '\n' )
        ++vllen;
    if( vllen == 1 )
        return pos + llen < getLength() ? pos + llen : getLength();
    if( vllen < lpos )
        return pos + llen + vllen - 1;
    return pos + llen + lpos;
}

bool Text::hat( Text *t ) const // enthlt der Text t
{
    bool ret = 0;
    if( t->getLength() > 0 ) // Auf unsinnige bergabe prfen
        ret = hat( t->getText() ); // prfen
    t = t->release(); // bergabe loslassen
    return ret;
}

bool Text::hat( const char *t ) const
{
    int tl = (int)strlen( t ); // Lnge der bergabe
    int txl = getLength(); // Lnge des Textes
    if( tl <= 0 || tl > txl ) // Auf unsinnige bergabe prfen
        return 0;
    int suchGCount = 0;
    for( int i = 0; i + tl <= txl; ++i ) // Text durchsuchen
    {
        if( !suchGCount )
        {
            bool b = 1;
            for( int ii = 0; ii < tl; ++ii ) // Text berprfen
                b &= txt[ i + ii ] == t[ ii ];
            if( b )
                return 1;
        }
        if( txt[ i ] == suchGBeg )
            ++suchGCount;
        if( txt[ i ] == suchGEnd )
            --suchGCount;
        if( !suchGCount )
        {
            bool b = 1;
            for( int ii = 0; ii < tl; ++ii ) // Text berprfen
                b &= txt[ i + ii ] == t[ ii ];
            if( b )
                return 1;
        }
    }
    return 0;
}

bool Text::hat( char c ) const // enthlt c
{
    int l = getLength(); // Text Lnge
    bool ret = 0;
    int suchGCount = 0;
    for( int i = 0; i < l; ++i ) // suchen
    {
        bool b = suchGCount != 0;
        if( !suchGCount ) // berprfen
            ret |= txt[ i ] == c;
        if( txt[ i ] == suchGBeg )
            ++suchGCount;
        if( txt[ i ] == suchGEnd )
            --suchGCount;
        if( !suchGCount && b ) // berprfen
            ret |= txt[ i ] == c;
    }
    return ret;
}

bool Text::istGleich( const char *t ) const // prft ob det Text gleich t ist
{
    int txl = getLength(); // Text Lnge
    int tl = (int)strlen( t ); // Lnge der bergabe
    if( txl != tl ) // Auf unsinniege bergabe prfen
        return 0;
    if( txl == -1 )
        return 1;
    bool ret = true;
    for( int i = 0; i < tl; ++i ) // prfen
        ret &= txt[ i ] == t[ i ];
    return ret;
}

bool Text::istGleich( Text *t ) const
{
    bool ret = istGleich( t->getText() ); // prfen
    t->release(); // bergabe loslassen
    return ret;
}

char *Text::getText() const // gibt Text zurck
{
    return txt;
}

int Text::anzahlVon( char c ) const // gibt die Anzahl von c im Text zurck
{
    int ret = 0;
    int l = getLength(); // Text Lnge
    int suchGCount = 0;
    for( int i = 0; i < l; ++i ) // suchen
    {
        bool b = suchGCount != 0;
        ret += txt[ i ] == c && !suchGCount; // zhlen
        if( txt[ i ] == suchGBeg )
            ++suchGCount;
        if( txt[ i ] == suchGEnd )
            --suchGCount;
        ret += txt[ i ] == c && !suchGCount && b; // zhlen
    }
    return ret;
}

int Text::anzahlVon( const char *t ) const // gibt die Anzahl von t im Text zurck
{
    int tl = (int)strlen( t ); // Lnge der bergabe
    int txl = getLength(); // Lnge des Textes
    if( tl <= 0 || tl > txl ) // Auf unsinnige bergabe prfen
        return 0;
    int ret = 0;
    int suchGCount = 0;
    for( int i = 0; i + tl <= txl; ++i ) // suchen
    {
        bool b = suchGCount != 0;
        if( !suchGCount )
        {
            bool b = 1;
            for( int ii = 0; ii < tl; ++ii ) // berprfen
                b &= txt[ i + ii ] == t[ ii ];
            if( b )
                ++ret; // zhlen
        }
        if( txt[ i ] == suchGBeg )
            ++suchGCount;
        if( txt[ i ] == suchGEnd )
            --suchGCount;
        if( !suchGCount && b )
        {
            bool b = 1;
            for( int ii = 0; ii < tl; ++ii ) // berprfen
                b &= txt[ i + ii ] == t[ ii ];
            if( b )
                ++ret; // zhlen
        }
    }
    return ret;
}

int Text::anzahlVon( Text *t ) const
{
    int ret = 0;
    if( t->getLength() > 0 )  // Auf unsinnige bergabe prfen
        ret = anzahlVon( t->getText() ); // zhlen
    t = t->release(); // bergabe loslassen
    return ret;
}

int Text::positionVon( char c ) const // gibt die Position des ersten c zurck
{
    int l = getLength(); // Text Lnge
    int suchGCount = 0;
    for( int i = 0; i < l; ++i ) // suchen
    {
        bool b = suchGCount != 0;
        if( txt[ i ] == c && !suchGCount ) // berprfen
            return i;
        if( txt[ i ] == suchGBeg )
            ++suchGCount;
        if( txt[ i ] == suchGEnd )
            --suchGCount;
        if( txt[ i ] == c && !suchGCount && b ) // berprfen
            return i;
    }
    return -1;
}

int Text::positionVon( const char *t ) const // gibt die Position des ersten t zurck
{
    int tl = (int)strlen( t ); // Lnge der bergabe
    int txl = getLength(); // Lnge des Textes
    if( tl <= 0 || tl > txl ) // Auf unsinnige bergabe prfen
        return -1;
    int suchGCount = 0;
    for( int i = 0; i + tl <= txl; ++i ) // suchen
    {
        bool b = suchGCount != 0;
        if( !suchGCount )
        {
            bool b = 1;
            for( int ii = 0; ii < tl; ++ii ) // berprfen
                b &= txt[ i + ii ] == t[ ii ];
            if( b )
                return i;
        }
        if( txt[ i ] == suchGBeg )
            ++suchGCount;
        if( txt[ i ] == suchGEnd )
            --suchGCount;
        if( !suchGCount && b )
        {
            bool b = 1;
            for( int ii = 0; ii < tl; ++ii ) // berprfen
                b &= txt[ i + ii ] == t[ ii ];
            if( b )
                return i;
        }
    }
    return -1;
}

int Text::positionVon( Text *t ) const
{
    int ret = 0;
    if( t->getLength() > 0 ) // Auf unsinnige bergabe prfen
        ret = positionVon( t->getText() ); // position ermitteln
    t = t->release(); // bergabe loslassen
    return ret;
}

int Text::positionVon( char c, int index ) const // gibt die Position des i-ten c zurck
{
    int l = getLength(); // Text Lnge
    int ii = 0;
    int suchGCount = 0;
    for( int i = 0; i < l; ++i ) // suchen
    {
        bool b = suchGCount != 0;
        if( txt[ i ] == c && !suchGCount ) // berprfen
        {
            if( ii == index )
                return i;
            else
                ++ii;
        }
        if( txt[ i ] == suchGBeg )
            ++suchGCount;
        if( txt[ i ] == suchGEnd )
            --suchGCount;
        if( txt[ i ] == c && !suchGCount && b ) // berprfen
        {
            if( ii == index )
                return i;
            else
                ++ii;
        }
    }
    return -1;
}

int Text::positionVon( const char *t, int index ) const // gibt die Position des i-ten t zurck
{
    int tl = (int)strlen( t ); // Lnge der bergabe
    int txl = getLength(); // Lnge des Textes
    if( tl <= 0 || tl > txl ) // Auf unsinnige bergabe prfen
        return 0;
    int i2 = 0;
    int suchGCount = 0;
    for( int i = 0; i + tl <= txl; ++i ) // suchen
    {
        bool b = suchGCount != 0;
        if( !suchGCount )
        {
            bool b = 1;
            for( int ii = 0; ii < tl; ++ii ) // berprfen
                b &= txt[ i + ii ] == t[ ii ];
            if( b )
            {
                if( i2 == index )
                    return i;
                else
                    ++i2;
            }
        }
        if( txt[ i ] == suchGBeg )
            ++suchGCount;
        if( txt[ i ] == suchGEnd )
            --suchGCount;
        if( !suchGCount && b )
        {
            bool b = 1;
            for( int ii = 0; ii < tl; ++ii ) // berprfen
                b &= txt[ i + ii ] == t[ ii ];
            if( b )
            {
                if( i2 == index )
                    return i;
                else
                    ++i2;
            }
        }
    }
    return -1;
}

int Text::positionVon( Text *t, int i ) const
{
    int ret = 0;
    if( t->getLength() > 0 ) // Auf unsinnige bergabe prfen
        ret = positionVon( t->getText(), i ); // position ermitteln
    t = t->release(); // bergabe loslassen
    return ret;
}

Text *Text::getTeilText( int p1, int p2 ) const // gibt den Text von p1 bis p2 zurck
{
    if( p1 > p2 ) // p1 und p2 tauschen
    {
        int x = p1;
        p1 = p2;
        p2 = x;
    }
    int l = getLength(); // Text Lnge
    if( p1 < 0 || p2 > l ) // Auf unsinnige bergabe prfen
        return new Text( "" );
    char *cp = new char[ p2 - p1 + 1 ]; // neuen Text erstellen
    for( int i = p1; i < p2; ++i ) // Text befllen
    {
        cp[ i - p1 ] = txt[ i ];
    }
    cp[ p2 - p1 ] = '\0';
    Text *t = new Text( cp ); // Text zurckgeben
    delete[]cp;
    return t;
}

Text *Text::getTeilText( int p ) const // gibt den Text von p bis zum Ende zurck
{
    return getTeilText( p, getLength() ); // Text zurckgeben
}

// Reference Counting
Text *Text::getThis()
{
    ++ref;
    return this;
}

Text *Text::release()
{
    --ref;
    if( ref < 1 )
        delete this;
    return 0;
}

// Operatoren
Text &Text::operator+=( const int num )
{
    append( num );
    return *this;
}

// Hngt eine Zahl ans Ende des Textes an
Text &Text::operator+=( const __int64 num )
{
    append( num );
    return *this;
}

Text &Text::operator+=( const double num )
{
    append( num );
    return *this;
}

Text &Text::operator+=( const char *txt )
{
    append( txt );
    return *this;
}

Text &Text::operator+=( const Text &txt )
{
    append( txt.getText() );
    return *this;
}

Text &Text::operator=( const int num )
{
    setText( "" );
    append( num );
    return *this;
}

Text &Text::operator=( const double num )
{
    setText( "" );
    append( num );
    return *this;
}

Text &Text::operator=( const char *txt )
{
    setText( txt );
    return *this;
}

Text &Text::operator=( const Text &txt )
{
    setText( txt.getText() );
    return *this;
}

Text::operator char*( ) const
{
    return txt;
}

Text::operator int() const
{
    if( getLength() > 2 && txt[ 0 ] == '0' && txt[ 1 ] == 'x' )
        return TextZuInt( ( txt + 2 ), 16 );
    return TextZuInt( txt, 10 );
}

Text::operator __int64() const
{
    if( getLength() > 2 && txt[ 0 ] == '0' && txt[ 1 ] == 'x' )
        return TextZuInt64( ( txt + 2 ), 16 );
    return TextZuInt64( txt, 10 );
}

Text::operator double() const
{
    if( getLength() > 2 && txt[ 0 ] == '0' && txt[ 1 ] == 'x' )
        return TextZuInt( ( txt + 2 ), 16 );
    return TextZuDouble( txt );
}

bool Text::operator>( Text &t ) const
{
    int len1 = getLength();
    int len2 = t.getLength();
    char *txt2 = t;
    for( int i = 0; i < len1 && i < len2; ++i )
    {
        if( txt[ i ] > txt2[ i ] )
            return 1;
        if( txt[ i ] < txt2[ i ] )
            return 0;
    }
    if( len1 > len2 )
        return 1;
    return 0;
}

bool Text::operator<( Text &t ) const
{
    int len1 = getLength();
    int len2 = t.getLength();
    char *txt2 = t;
    for( int i = 0; i < len1 && i < len2; ++i )
    {
        if( txt[ i ] < txt2[ i ] )
            return 1;
        if( txt[ i ] > txt2[ i ] )
            return 0;
    }
    if( len1 < len2 )
        return 1;
    return 0;
}

// Inhalt der TextReader Klasse
// Konstructor
//  txt: Der Text der gelesen werden soll. Er wird nicht kopiert sondern direkt gelesen.
TextReader::TextReader( Text *txt )
{
    this->txt = txt;
    lPos = 0;
    ref = 1;
}

// Destructor
TextReader::~TextReader()
{
    txt->release();
}

// Setzt die Position des Bytes, das als nchstes gelesen werden soll
//  pos: Der Index des Bytes
//  ende: 1, wenn der Index vom ende des Textes zhlt. 0, wenn der Index vom Beginn des Textes zhlt
void TextReader::setLPosition( __int64 pos, bool ende )
{
    int l = txt->getLength();
    lPos = ende ? l - pos : pos;
    if( lPos < 0 )
        lPos = 0;
    if( lPos > l )
        lPos = l;
}

// Liet aus der Datei
//  bytes: Ein Array, der mit Bytes aus dem Text gefllt werden soll
//  len: Wie viele Bytes aus dem Text gelesen werden sollen
void TextReader::lese( char *bytes, int len )
{
    int l = txt->getLength();
    len = (int)MIN( l - lPos, len );
    for( __int64 i = lPos; i < lPos + len; i++ )
        bytes[ i - lPos ] = txt->getText()[ i ];
    lPos += len;
}

// Liet die nchste zeile des Textes ein
//  return: Die gelesene Zeile als Text mit zeilenumbruch
Text *TextReader::leseZeile()
{
    if( istEnde() )
        return 0;
    Text *ret = new Text( "" );
    __int64 len = txt->getLength();
    for( char c = 0; c != '\n' && lPos < len; )
    {
        lese( &c, 1 );
        if( c )
            ret->append( (const char*)&c, 1 );
    }
    return ret;
}

// Prft, ob die Resource vollstndig gelesen wurde
//  return 1, wenn die Resource vollstndig gelesen wurde. 0, sonst
bool TextReader::istEnde() const
{
    return lPos >= txt->getLength();
}

// Gibt den Index des Bytes aus dem Text zurck, welches als nchstes gelesen werden wrde
// return -1, falls ein Fehler aufgetreten ist. Sonst die Position des Lesezeigers
__int64 TextReader::getLPosition() const
{
    return lPos;
}

// Erhht den Reference Counting Zhler.
//  return: this.
TextReader *TextReader::getThis()
{
    ref++;
    return this;
}

// Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
//  return: 0.
TextReader *TextReader::release()
{
    if( !--ref )
        delete this;
    return 0;
}

// char* operationen
int Framework::stringPositionVonChar( char *string, char c, int num ) // sucht die position des num-ten c-s in string, -1 bei nicht gefunden
{
    int gef = 0;
    int p = 0;
    for( char cc = *string; *string; ++string )
    {
        if( cc == c )
        {
            if( gef == num )
                return p;
            else
                ++gef;
        }
        ++p;
    }
    return -1;
}

int Framework::stringPositionVonString( char *string, char *suche, int sBegPos ) // sucht die position von 'suche' in 'string' ab der Position 'sBegPos', -1 bei nicht gefunden
{
    for( int i = 0; i < sBegPos; ++i )
    {
        if( !*string )
            return -1;
        ++string;
    }
    int tl = textLength( suche ); // Lnge der bergabe
    int txl = textLength( string ); // Lnge des Textes
    if( tl <= 0 || tl > txl ) // Auf unsinnige bergabe prfen
        return -1;
    for( int i = 0; i + tl <= txl; ++i ) // suchen
    {
        bool b = 1;
        for( int ii = 0; ii < tl; ++ii ) // berprfen
            if( b )
                b = string[ i + ii ] == suche[ ii ];
        if( b )
            return i + sBegPos;
    }
    return -1;
}

//---Andere Funktionen---
void Framework::TextKopieren( const char *txt ) // kopiert den Text in den Zwischenspeicher
{
#ifdef WIN32
    int ln = textLength( txt ) + 1;
    if( ln == 1 )
        return;
    HGLOBAL hMem = GlobalAlloc( GMEM_MOVEABLE, ln );
    if( !hMem )
        return;
    memcpy( GlobalLock( hMem ), txt, ln );
    GlobalUnlock( hMem );
    OpenClipboard( 0 );
    EmptyClipboard();
    SetClipboardData( CF_TEXT, hMem );
    CloseClipboard();
#endif
}

char *Framework::TextInsert() // gibt den Text aus der Zwischenablage zurck
{
#ifdef WIN32
    if( !OpenClipboard( 0 ) )
        return "";
    HANDLE hClipData = GetClipboardData( CF_TEXT );
    char *cBuffer = (char*)GlobalLock( hClipData );
    GlobalUnlock( hClipData );
    CloseClipboard();
    return cBuffer;
#else
    return 0;
#endif
}

char Framework::smallOrBig( char c, bool gr )
{
    int ret = c;
    if( gr )
    {
        if( c >= 'a' && c <= 'z' )
            ret -= 32;
        else
        {
            switch( c )
            {
            case '^':
                return '';
            case '1':
                return '!';
            case '<':
                return '>';
            case '2':
                return '\"';
            case '3':
                return '';
            case '4':
                return '$';
            case '5':
                return '%';
            case '6':
                return '&';
            case '7':
                return '/';
            case '8':
                return '(';
            case '9':
                return ')';
            case '0':
                return '=';
            case ',':
                return ';';
            case '.':
                return ':';
            case '':
                return '?';
            case '-':
                return '_';
            case '':
                return '`';
            case '+':
                return '*';
            case '#':
                return '\'';
            case '':
                return '';
            case '':
                return '';
            case '':
                return '';
            }
        }
    }
    else
    {
        if( c >= 'A' && c <= 'Z' )
            ret += 32;
        else
        {
            switch( c )
            {
            case '':
                return '^';
            case '!':
                return '1';
            case '>':
                return '<';
            case '\"':
                return '2';
            case '':
                return '3';
            case '$':
                return '4';
            case '%':
                return '5';
            case '&':
                return '6';
            case '/':
                return '7';
            case '(':
                return '8';
            case ')':
                return '9';
            case '=':
                return '0';
            case ';':
                return ',';
            case ':':
                return '.';
            case '?':
                return '';
            case '_':
                return '-';
            case '`':
                return '';
            case '*':
                return '+';
            case '\'':
                return '#';
            case '':
                return '';
            case '':
                return '';
            case '':
                return '';
            }
        }
    }
    return (char)ret;
}

bool Framework::istSchreibbar( unsigned char zeichen ) // prft, ob zeichen ein Schreibbarer Buchstabe ist
{
    if( zeichen > 32 && zeichen < 127 )
        return 1;
    if( zeichen == 128 || zeichen == 181 || zeichen == 178 || zeichen == 179 )
        return 1;
    if( zeichen > 191 && zeichen < 198 )
        return 1;
    if( zeichen > 199 && zeichen < 208 )
        return 1;
    if( zeichen > 209 && zeichen < 215 )
        return 1;
    if( zeichen > 216 && zeichen < 221 )
        return 1;
    if( zeichen > 222 && zeichen < 230 )
        return 1;
    if( zeichen > 231 && zeichen < 240 )
        return 1;
    if( zeichen > 241 && zeichen < 247 )
        return 1;
    if( zeichen > 248 && zeichen < 253 )
        return 1;
    if( zeichen == ' ' || zeichen == '\t' )
        return 1;
    return 0;
}

unsigned int Framework::TextZuInt( char *c, int system ) // Konvertiert c zu int
{
    if( system == 16 )
        return (unsigned int)strtoul( c, 0, system );
    return (unsigned int)strtol( c, 0, system );
}

unsigned int Framework::TextZuInt( char *c, char **c_ende, int system )
{
    if( system == 16 )
        return (unsigned int)strtoul( c, c_ende, system );
    return (unsigned int)strtol( c, c_ende, system );
}

unsigned __int64 Framework::TextZuInt64( char *c, int system )
{
    if( system == 16 )
        return strtoull( c, 0, system );
    return (unsigned __int64)strtoll( c, 0, system );
}

unsigned __int64 Framework::TextZuInt64( char *c, char **c_ende, int system )
{
    if( system == 16 )
        return strtoull( c, c_ende, system );
    return (unsigned __int64)strtoll( c, c_ende, system );
}

double Framework::TextZuDouble( char *c ) // Konvertiert c zu double
{
    return strtod( c, 0 );
}

double Framework::TextZuDouble( char *c, char **c_ende )
{
    return strtod( c, c_ende );
}

int Framework::textLength( const char *txt ) // gibt die Lnge von txt zurck
{
    if( !txt )
        return 0;
    int ret = 0;
    for( ; txt[ ret ] != 0; ++ret );
    return ret;
}